// File: Document.pas
//
// Description: Implementation of the TDocProperties class
// containing print job information

unit Document;

interface

uses SysUtils, IniFiles;

type
  // Page orientation
  TPageOrientation = (poPortrait, poLandscape);

  // TDocProperties class
  TDocProperties = class
  private
    // Print job parameters
    FDeviceName: string;                // Device (printer) name
    FDocumentName: string;              // Printed document name
    FPrintJobId: Integer;               // Print job ID
    FMachineName: string;               // Machine name
    FUserName: string;                  // User name
    FSessionId: Integer;                // Session ID
    FHorizontalResolution: Integer;     // Horizontal resolution (dpi)
    FVerticalResolution: Integer;       // Vertical resolution (dpi)
    FPageOrientation: TPageOrientation; // Page orientation (Portrait, Landscape)
    FPaperSize: Integer;                // Paper size (index)
    FPaperSizeName: string;             // Paper size name
    FPaperWidth: Integer;               // Paper width
    FPaperLength: Integer;              // Paper length
    FPagesQuantity: Integer;            // Amount of pages
    FCopiesQuantity: Integer;           // Amount of copies
    FCollate: Boolean;                  // Collate option
    FPaperBin: Integer;                 // Paper Bin (index)
    FPaperBinName: string;              // Paper Bin name
    FFiles: array of string;            // Array of .emf file names (pages)

    function GetFiles(Index: Integer): string;

    // Set all parameters to zero or empty string
    procedure ClearAll(const DoClearStrings: Boolean);
  public
    property DeviceName: string read FDeviceName;
    property DocumentName: string read FDocumentName;
    property PrintJobId: Integer read FPrintJobId;
    property MachineName: string read FMachineName;
    property UserName: string read FUserName;
    property SessionId: Integer read FSessionId;
    property HorizontalResolution: Integer read FHorizontalResolution;
    property VerticalResolution: Integer read FVerticalResolution;
    property PageOrientation: TPageOrientation read FPageOrientation;
    property PaperSize: Integer read FPaperSize;
    property PaperSizeName: string read FPaperSizeName;
    property PaperWidth: Integer read FPaperWidth;
    property PaperLength: Integer read FPaperLength;
    property PagesQuantity: Integer read FPagesQuantity;
    property CopiesQuantity: Integer read FCopiesQuantity;
    property Collate: Boolean read FCollate;
    property PaperBin: Integer read FPaperBin;
    property PaperBinName: string read FPaperBinName;
    property Files[Index: Integer]: string read GetFiles;

    // Retrieve information from the specified .ini file
    function Retrieve(const IniFileName: string): Boolean;
  end;

implementation

function TDocProperties.GetFiles(Index: Integer): string;
begin
  if (Index < 0) or (Index > High(FFiles)) then
    raise Exception.Create('Index out of bounds')
  else
    Result := FFiles[Index];
end;

// Set all parameters to zero or empty string
procedure TDocProperties.ClearAll(const DoClearStrings: Boolean);
begin
  if DoClearStrings then
  begin
    FDeviceName := '';
    FDocumentName := '';
    FMachineName := '';
    FUserName := '';
    FPaperSizeName := '';
    FPaperBinName := '';
    FFiles := nil;
  end;

  FPrintJobId := 0;
  FSessionId := 0;
  FHorizontalResolution := 0;
  FVerticalResolution := 0;
  FPaperSize := 0;
  FPaperWidth := 0;
  FPaperLength := 0;
  FPagesQuantity := 0;
  FCopiesQuantity := 0;
  FPaperBin := 0;

  FPageOrientation := poPortrait;
  FCollate := false;
end;

// Retrieve information from the specified .ini file
function TDocProperties.Retrieve(const IniFileName: string): Boolean;
var
  Ini: TIniFile;
  i: Integer;
begin
  Result := false;
	ClearAll(true);

  if FileExists(IniFileName) then
  begin
    Ini := TIniFile.Create(ExpandFileName(IniFileName));

    FDeviceName := Ini.ReadString('Device', 'DeviceName', 'Unknown');
    FDocumentName := Ini.ReadString('Document', 'Name', '');
    FPrintJobId := Ini.ReadInteger('Document', 'JobID', 0);
    FMachineName := Ini.ReadString('Document', 'MachineName', '');
    FUserName := Ini.ReadString('Document', 'UserName', '');
    FSessionId := Ini.ReadInteger('Document', 'SessionID', 0);
    FHorizontalResolution := Ini.ReadInteger('Document', 'HorizontalResolution', 0);
    FVerticalResolution := Ini.ReadInteger('Document', 'VerticalResolution', 0);
    i := Ini.ReadInteger('Document', 'Orientation', 1);
    if i = 1 then FPageOrientation := poPortrait else FPageOrientation := poLandscape;
    FPaperSize := Ini.ReadInteger('Document', 'PaperSize', 0);
    FPaperSizeName := Ini.ReadString('Document', 'PaperSizeName', 'Unknown');
    FPaperWidth := Ini.ReadInteger('Document', 'PaperWidth', 0);
    FPaperLength := Ini.ReadInteger('Document', 'PaperLength', 0);
    FPagesQuantity := Ini.ReadInteger('Document', 'Pages', 0);
    FCopiesQuantity := Ini.ReadInteger('Document', 'Copies', 0);
    FCollate := Ini.ReadInteger('Document', 'Collate', 0) <> 0;
    FPaperBin := Ini.ReadInteger('Document', 'Bin', 0);
    FPaperBinName := Ini.ReadString('Document', 'BinName', 'Unknown');

    SetLength(FFiles, FPagesQuantity);
    for i := 0 to FPagesQuantity - 1 do
      FFiles[i] := Ini.ReadString('EMF', Format('File%d', [i]), '');

    Ini.Free;
    Result := true;
  end;
end;

end.