// File: Main.pas
//
// Description: Main application form demonstrating virtual printer integration
// methods, print job information retrieval and presentation

unit Main;

interface

uses Windows, Messages, SysUtils, Classes, Controls, Forms, StdCtrls, ExtCtrls,
  Types, Graphics, ShellApi, Document;

type
  // Main form class
  TMainForm = class(TForm)
    PreviewLabel: TLabel;
    PreviewBevel: TBevel;
    PreviewImage: TImage;
    Label1: TLabel;
    ListBox: TListBox;
    Bevel1: TBevel;
    Label2: TLabel;
    CloseButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure PreviewImageDblClick(Sender: TObject);
    procedure ListBoxClick(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
  private
    FDocProps: TDocProperties;
    FListBoxData: array of Integer;
    FCurrentPage: Integer;
    procedure FillListBox;
    procedure LoadPreview(const PageIndex: Integer = -1;
      const DoRepaint: Boolean = true);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

// This functions fills in the ListBox (print job details)
procedure TMainForm.FillListBox;
var
  i, x: Integer;
  s: string;
begin
  ListBox.Items.Add('Document name: ' + FDocProps.DocumentName);
  ListBox.Items.Add(Format('Print job ID: %d', [FDocProps.PrintJobId]));
  ListBox.Items.Add('');

  ListBox.Items.Add(Format('Horizontal resolution: %d dpi',
    [FDocProps.HorizontalResolution]));
  ListBox.Items.Add(Format('Vertical resolution: %d dpi',
    [FDocProps.VerticalResolution]));
  ListBox.Items.Add('');

  ListBox.Items.Add(Format('Paper size: %s (%d x %d mm)', [FDocProps.PaperSizeName,
    FDocProps.PaperWidth div 10, FDocProps.PaperLength div 10]));
  ListBox.Items.Add('Paper bin: ' + FDocProps.PaperBinName);
  if FDocProps.PageOrientation = poPortrait then s := 'Portrait' else s := 'Landscape';
  ListBox.Items.Add('Orientation: ' + s);
  ListBox.Items.Add('');

  ListBox.Items.Add(Format('Pages: %d', [FDocProps.PagesQuantity]));
  ListBox.Items.Add(Format('Copies: %d', [FDocProps.CopiesQuantity]));
  if FDocProps.Collate then s := 'Yes' else s := 'No';
  ListBox.Items.Add('Collate: ' + s);
  ListBox.Items.Add('');

  if FDocProps.PagesQuantity = 0 then
    ListBox.Items.Add('There are no printed pages')
  else begin
    ListBox.Items.Add('Printed pages (click to preview):');
    for i := 0 to FDocProps.PagesQuantity - 1 do
    begin
      // Add file name to the list
      x := ListBox.Items.Add(Format('    Page %d: %s',
        [i+1, ExtractFileName(FDocProps.Files[i])]));

      // Mark the listbox row as "clickable", its item data contains
      // index of the .emf file in the m_DocProps.m_Files array
      SetLength(FListBoxData, ListBox.Items.Count);
      FListBoxData[x] := $8000 + i;

      // Select 1st page in the list
      if i = 0 then ListBox.ItemIndex := x;
    end;
  end;
end;

// This function loads the specified page to the preview panel
procedure TMainForm.LoadPreview(const PageIndex: Integer = -1;
  const DoRepaint: Boolean = true);
var
  EmfPage: HENHMETAFILE;
  R: TRect;
  DC: HDC;
begin
  // Resize preview image if needed
  PreviewImage.Picture.Bitmap.Width := PreviewImage.Width;
  PreviewImage.Picture.Bitmap.Height := PreviewImage.Height;
  PreviewImage.Picture.Bitmap.PixelFormat := pf24bit;

  // Load the specified emf page
  EmfPage := 0;
  if PageIndex >= 0 then
    EmfPage := GetEnhMetaFile(PChar(FDocProps.Files[PageIndex]));

  // Get DC handle
  DC := PreviewImage.Canvas.Handle;
  R := Rect(0, 0, PreviewImage.Width, PreviewImage.Height);
  BitBlt(DC, 0, 0, R.Right, R.Bottom, 0, 0, 0, WHITENESS);

  if EmfPage = 0 then
  begin
    // Draw 'No image' if no preview is loaded
    DrawText(DC, 'No image', -1, R, DT_CENTER or DT_SINGLELINE or DT_VCENTER);

    FCurrentPage := -1;

    // Update preview's label
    PreviewLabel.Caption := 'Page preview:';
  end else begin
    // Fix rectangle for landscape orientation
    if FDocProps.PageOrientation = poLandscape then
      R.Bottom := R.Bottom div 2;

    // Render .emf file to image canvas
    PlayEnhMetaFile(DC, EmfPage, R);

    FCurrentPage := PageIndex;

    // Update preview's label
    PreviewLabel.Caption := Format('Page preview (page %d):', [PageIndex + 1]);
  end;

  // Delete loaded emf page
  if EmfPage <> 0 then
    DeleteEnhMetaFile(EmfPage);

  // Update preview image
  if DoRepaint then
    PreviewImage.Repaint;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.Title := Caption;

  // Create FDocProps object
  FDocProps := TDocProperties.Create;

  // Initialize page preview
  LoadPreview;

  // Parse .ini file
  if not FDocProps.Retrieve(ParamStr(1)) then
    ListBox.Items.Add('Invalid command line parameter!')
  else begin
    // Fill in the ListBox
    FillListBox;

    // Load 1st page preview
    if FDocProps.PagesQuantity > 0 then
      LoadPreview(0);
  end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  FDocProps.Free;
  FListBoxData := nil;
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  // Reload page preview
  LoadPreview(FCurrentPage, false);
end;

procedure TMainForm.PreviewImageDblClick(Sender: TObject);
begin
  // Open page preview with the default application
  if FCurrentPage >= 0 then
    ShellExecute(Handle, 'open', PChar(FDocProps.Files[FCurrentPage]),
      nil, nil, SW_SHOWNORMAL);
end;

procedure TMainForm.ListBoxClick(Sender: TObject);
var
  Page, x: Integer;
begin
  // If the clicked item of the listbox corresponds to a printed page
  // then appropriate .emf file will be opened on the preview panel
  x := ListBox.ItemIndex;
  if (x >= 0) and (x < Length(FListBoxData)) then
    if FListBoxData[x] and $8000 > 0 then
    begin
      Page := FListBoxData[x] and $7FFF;
      if Page <> FCurrentPage then
        LoadPreview(Page);
    end;
end;

procedure TMainForm.CloseButtonClick(Sender: TObject);
begin
  Close;
end;

end.
